#!/usr/bin/env python
__version__ = '$Revision: 2275 $'.split()[1]
__date__ = '$Date: 2006-07-10 16:22:35 -0400 (Mon, 10 Jul 2006) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''
Connect to a socket and forward what is received to another port.
Filter to a list of AIS receivers/basestations.

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@copyright: 2006
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@status: under development
@license: GPL v2
@since: Jan 2008

@todo: Split send and receive into two separate threads with a queue
in between so that we do not loose input data when dest is down for a bit
@todo: If dest is down for too long, start dropping messages if the queue gets to long

'''

#import shutil
#import os
import sys
import time
import socket
import thread
import select
import aisutils.uscg

######################################################################

def dateStr():
    '''
    String representing the day so that it sorts correctly
    @return: yyyy-mm-dd
    @rtype: str
    '''
    t=time.gmtime()
    d='%04d-%02d-%02d' % t[:3]
    return d


######################################################################
class PassThroughServer:
    '''Receive data from a socket and write the data to all clients that
    are connected.  Starts two threads and returns to the caller.
    '''
    def __init__(self,options):
	self.clients=[]
	self.options = options
	self.count=0
        self.running=True
        self.timeout=options.timeout

    def start(self):
	print 'starting thread'
	thread.start_new_thread(self.passdata,(self,))
	return

    def passdata(self,unused=None):
	'''Do not use this.  Call start() instead.

	@bug: how can I get rid of unused?
	'''
	print 'starting passthrough server'

	# input - where our data comes from
        connected=False
        count=0
        while not connected:
            count += 1
            try:
                src = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
                src.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                src.connect((self.options.inHost,self.options.inPort))
            except socket.error, inst:
                sys.stderr.write(str(count)+': Failed to connect to src ... '+str(inst)+'\tWill try again\n')
                time.sleep(5)
            else:
                connected=True
        del connected

	# dst is where our data comes from
        connected=False
        count = 0
        while not connected:
            count += 1
            try:
                dst = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
                dst.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                dst.connect((self.options.outHost,self.options.outPort))
            except socket.error, inst:
                sys.stderr.write(str(count)+': Failed to connect to dst ... '+str(inst)+'\t\tWill try again\n')
                time.sleep(5)
            else:
                connected=True
        del connected

	while self.running:
            sys.stderr.write('calling select\n')
	    #readersready,outputready,exceptready = select.select([src,],[dst,],[],self.timeout)
	    readersready,outputready,exceptready = select.select([src,],[],[],self.timeout)
	    # FIX: how to detect dst disconnect?
	    for sock in readersready:
                sys.stderr.write('top of for loop\n')
                m = sock.recv(10000)
                if len(m)==0:
                    srcConnected=False
                    count=0
                    while not srcConnected:
                        count += 1
                        sys.stderr.write('trying to connect to src ...\n')
                        try:
                            src = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
                            src.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                            src.connect((self.options.inHost,self.options.inPort))
                        except socket.error, inst:
                            sys.stderr.write(str(count)+': Failed to reconnect to src ... '+str(inst)+'\tWill try again\n')
                            time.sleep(5)
                            sys.stderr.write('done sleeping\n')
                        else:
                            srcConnected=True
                    sys.stderr.write('reconnected to src\n')
                sys.stderr.write('done if len(0)\n')
                if len(m)==0: 
                    sys.stderr.write('calling continue\n')
                    continue

                sys.stderr.write('splitting messages\n')
                msgs = m.split('\n')
                toSend=[]
                for msg in msgs:
                    if len(msg)<6: continue
                    if 'AIVDM'!=msg[1:6]:
                        print 'not aivdm',msg
                        continue
                    sys.stderr.write('parsing station from: ' + msg + '\n')
                    station = aisutils.uscg.UscgNmea(msg).station
                    if station in self.options.allowStations:
                        toSend.append(msg)
                        sys.stderr.write('forwarding '+msg+'\n')
                    else:
                        sys.stderr.write('rejecting station '+station+'\n')
                if 0==len(toSend):
                    sys.stderr.write('Nothing to send...\n')
                    continue

                m='\n'.join(toSend)
                sys.stderr.write('Sending '+str(m)+'\n')
                sent = False
                while not sent:
                    try:
                        #print 'skipping send'
                        dst.send(m)
                        sent=True
                    except socket.error:
                        sys.stderr.write('Destination disconnect\n')
                        sys.stderr.write('what happens now?\n')
                    if not sent:
                        sys.stderr.write('Trying to reconnect to dst:'
                                         +str(self.options.outHost)
                                         +':'
                                         +str(self.options.outPort)+'\n')
                        dst = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
                        dst.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                        dst.connect((self.options.outHost,self.options.outPort))
                    
        sys.stderr.write('end of thread\n')


######################################################################


if __name__=='__main__':
    from optparse import OptionParser

    # FIX: is importing __init__ safe?
    parser = OptionParser(usage="%prog [options]"
                          ,version="%prog "+__version__ + " ("+__date__+")")

    parser.add_option('-i','--in-port',dest='inPort',type='int', default=31401
                      ,help='Where the data comes from [default: %default]')
    parser.add_option('-I','--in-host',dest='inHost',type='string',default='localhost'
                      ,help='What host to read data from [default: %default]')
#    myparser.add_option('--in-gethostname',dest='inHostname', action='store_true', default=False
#			help='Where the data comes from [default: %default]')


    parser.add_option('-o','--out-port', dest="outPort", type='int',default=31402
                      ,help='Where to send the data [default: %default]')
    parser.add_option('-O','--out-host',dest='outHost',type='string', default='localhost'
                      ,help='What machine to send the data to [default: %default]')
    parser.add_option('--out-gethostname',dest='outHostname', action='store_true', default=False
                      ,help='Use the default hostname ['+socket.gethostname()+']')

    parser.add_option('-t','--timeout',dest='timeout',type='float', default='300'
                      ,help='Number of seconds to timeout after if no data [default: %default]')

    # FIX: Change to allow the user to specify when logs will be rotated
#    myparser.add_option('-r','--rotate', dest='rotateLog', default=False,
#			action='store_true', help='turn on one a day log rotation.'+
#			'Appends the date to the log')


    parser.add_option('-a','--add-station',action='append',dest='allowStations'
                      ,default=None
                      ,help='Specify limited set stations to forward (e.g. r003679900) [default: all]')


    parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true'
                      ,help='Make the test output verbose')

    (options,args) = parser.parse_args()

    if options.outHostname:
	options.outHost=socket.gethostname()

    pts = PassThroughServer(options)
    pts.start()
    
    v = options.verbose
    del(options) # remove global to force self.options
    i = 0
    while 1:
	i+=1
	time.sleep(30)
	if v: print 'ping',i
